# 通知消息事件
NOTICE_MESSAGE = 10000
# 系统消息事件
SYSTEM_MESSAGE = 10002
# 全部消息事件
ALL_MESSAGE = 99999
# 文本消息事件
TEXT_MESSAGE = 1
# 图片消息事件
IMAGE_MESSAGE = 3
# 语音消息事件
VOICE_MESSAGE = 34
# 好有验证请求消息事件
FRIEND_VERIFY_MESSAGE = 37
# 卡片消息事件
CARD_MESSAGE = 42
# 视频消息事件
VIDEO_MESSAGE = 43
# 表情消息事件
EMOJI_MESSAGE = 47
# 位置消息事件
LOCATION_MESSAGE = 48
# xml消息事件
XML_MESSAGE = 49
# 视频/语音通话消息事件
VOIP_MESSAGE = 50
# 手机端同步消息事件
PHONE_MESSAGE = 51